@@ -1,6 +1,7 @@
 from enum import Enum
 from abc import ABC, abstractmethod
 from coopstructs.vectors import Vector2
+from cooptools.geometry import collinear_points
 import math
 from typing import List, Callable, Dict
 from coopstructs.toggles import EnumToggleable
@@ -427,19 +428,38 @@ class CubicBezier(Curve):
 
         # handle check if circle is a good approximater
         if curve.origin != curve.end_point:
-            circle, deviation = curve.biarc_circle()
+            circle= curve.biarc_circle()
+            deviation = cls.max_delta_on_circle(curve, circle) if circle is not None else None
             if circle is None:
-                pass
+                # no arcs for this curve since the circle could not be defined (straight line bezier)
+                return []
             elif deviation < tolerance:
+                # approximating arc is withing tolerance, return the arc
                 return [CircularArc.from_3_consecutive_points(a=circle.known_boundary_points[0],
                                                               b=circle.known_boundary_points[1],
                                                               c=circle.known_boundary_points[2],
                                                               naming_provider=lambda: str(uuid4()))]
 
-        # handle subdivide and try again
-        sub_divisions = curve.sub_divide_at_t(0.5)
-        arcs = [CubicBezier.as_circular_arcs(x, tolerance) for x in sub_divisions]
-        return [item for sublist in arcs for item in sublist]
+            # handle subdivide and try again
+            sub_divisions = curve.sub_divide_at_t(0.5)
+            arcs = [CubicBezier.as_circular_arcs(x, tolerance) for x in sub_divisions]
+            return [item for sublist in arcs for item in sublist]
+        else:
+            return []
+
+    @classmethod
+    def max_delta_on_circle(cls, curve, biarc_circle: Circle):
+        max_delta = 0
+        for point in curve.compute_bezier_points([(x.x, x.y) for x in curve.control_points], 10):
+            point = Vector2(point[0], point[1])
+            dist_from_center = (point - biarc_circle.center).length()
+            deviation = abs(biarc_circle.radius - dist_from_center)
+            if deviation > max_delta:
+                max_delta = deviation
+
+        return max_delta
+
+
 
     @classmethod
     def from_points(cls, points: List[Vector2], naming_provider: Callable[[], str]) -> (List, List):
@@ -559,6 +579,7 @@ class CubicBezier(Curve):
         else:
             return next_point.project_onto(self.control_points[-1], self.control_points[-2])
 
+
     def biarc_circle(self) -> (Circle, float):
         p1 = self.control_points[0]
         p4 = self.control_points[3]
@@ -572,22 +593,17 @@ class CubicBezier(Curve):
         try:
             V = line1.intersection(line2)
         except:
-            return None, None
+            return None
 
-        tri = Triangle(p1, V, p4)
-        incentre = tri.incentre()
+        if not collinear_points([x.as_tuple() for x in [p1, V, p4]]):
+            tri = Triangle(p1, V, p4)
+            incentre = tri.incentre()
 
-        biarc_circ = Circle.from_boundary_points(p1, incentre, p4)
+            biarc_circ = Circle.from_boundary_points(p1, incentre, p4)
+        else:
+            return None
 
-        max_delta = 0
-        for point in self.compute_bezier_points([(x.x, x.y) for x in self.control_points], 10):
-            point = Vector2(point[0], point[1])
-            dist_from_center = (point - biarc_circ.center).length()
-            deviation = abs(biarc_circ.radius - dist_from_center)
-            if deviation > max_delta:
-                max_delta = deviation
-
-        return biarc_circ, max_delta
+        return biarc_circ
 
     def sub_divide_at_t(self, t: float):
         if not (0 < t < 1):
@@ -608,7 +624,7 @@ class CubicBezier(Curve):
                 CubicBezier(id=str(uuid4()), control_points=[point_t, s2, s3, p4])]
 
     def point_at_t(self, t: float):
-        if not (0 < t < 1):
+        if not (0 <= t <= 1):
             raise ValueError(f"t must be between 0 and 1 but {t} was provided")
 
         b0x = self.control_points[0].x
@@ -649,7 +665,14 @@ class CubicBezier(Curve):
         f_1_x = lambda t: a_x * (1 - t) ** 2 + 2 * b_x * (1 - t) * t + c_x * t ** 2
         f_1_y = lambda t: a_y * (1 - t) ** 2 + 2 * b_y * (1 - t) * t + c_y * t ** 2
 
-        ret = Vector2(f_1_x(t), f_1_y(t))
+        if t == 1 and b3x == b2x and b3y == b2y:
+            ret = Vector2(b3x - b1x, b3y - b1y)
+        elif t == 1:
+            ret = Vector2(b3x - b2x, b3y - b2y)
+        elif t == 0:
+            ret = Vector2(b1x - b0x, b1y - b0y)
+        else:
+            ret = Vector2(f_1_x(t), f_1_y(t))
         return ret
 
     def inflection_points(self):
@@ -826,14 +849,18 @@ class CurveBuilder:
         self.current_curve_type.toggle()
         self.current_points = []
 
+    def saved_and_temp_from_next_points(self, next_points: List[Vector2]):
+        curves = [curve for id, curve in self.curves.items()]
+        curves += self.curves_from_temp_next_points(next_points)
+        return curves
+
     def save_curve(self):
         new_curves, _ = self.curves_from_points(self.current_curve_type.value, self.current_points)
 
         new_curve_dict = {}
         for curve in new_curves:
-            new_id = self.id_provider()
-            new_curve_dict[new_id] = curve
-            self.curves[new_id] = curve
+            new_curve_dict[curve.id] = curve
+            self.curves[curve.id] = curve
         self.current_points = []
 
         return new_curve_dict
@@ -892,10 +919,14 @@ class CurveBuilder:
             return next_point
 
         # Dont need to force a continuity if we are in the middle of a Bezier
-        if self.current_curve_type.value == CurveType.CUBICBEZIER and len(leftovers) != 0:
-            return next_point
-        elif close_circuit:
+        #       For closing a circuit, need to identify the second to last point of 4 points of the Bezier. Since we
+        #       progress through a list of cubic beziers, this means that the end of the last curve is point 1, if there
+        #       is only one leftover point, it is point 2. Meaning that we will be choosing the 3rd (second to last)
+        #       point when there is one leftover point returned from the curve
+        if close_circuit and len(leftovers) == 1:
             return curves[0].force_continuity(next_point, close_circuit)
+        elif len(leftovers) != 0:
+            return next_point
         else:
             return curves[-1].force_continuity(next_point)
 
