@@ -90,6 +90,8 @@ class IntegerRangeToggleable(Toggleable):
         return type(self)(min=self.min, max=self.max, step_size=self.step)
 
 class EnumToggleable(Toggleable):
-    def __init__(self, enum_type):
+    def __init__(self, enum_type, default=None):
         self.enum = enum_type
-        Toggleable.__init__(self, [val for val in enum_type.__members__.values()], 0)
\ No newline at end of file
+        Toggleable.__init__(self, [val for val in enum_type.__members__.values()], 0)
+        if default is not None:
+            self.set_value(default)
\ No newline at end of file
