@@ -1,6 +1,7 @@
 import math
 from typing import Dict, List, Tuple
 import numpy as np
+import cooptools.geometry as geo
 
 class IVector:
     def from_ivector(self, ivec):
@@ -341,6 +342,9 @@ class Vector2 (IVector):
         self.coords['x'] = x
         self.coords['y'] = y
 
+    def in_polygon(self, poly: List[IVector]):
+        return geo.point_in_polygon(self.as_tuple(), [x.as_tuple()[:2] for x in poly])
+
 class Vector3(IVector):
     @classmethod
     def from_ivector(cls, ivec: IVector):
@@ -371,6 +375,16 @@ class Vector3(IVector):
         self.coords['z'] = z
 
 if __name__ == "__main__":
-    a = Vector2(1, 2)
+    poly = [
+        Vector2(2, 2),
+        Vector2(3, 3),
+        Vector2(2, 3),
+        Vector2(3, 2)
+    ]
 
-    print (a * 4)
\ No newline at end of file
+    point = Vector2(2.5, 2.5)
+
+    print(point.in_polygon(poly))
+
+    point = Vector2(1, 1)
+    print(point.in_polygon(poly))
\ No newline at end of file
